@@ -13,7 +13,7 @@
  *
  *   http://alexandre.alapetite.net/doc-alex/domxml-php4-php5/
  *
- * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.3 2005/02/06 04:25:00 chuck Exp $
+ * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.7 2005/03/25 11:10:16 jan Exp $
  *
  * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
  *
@@ -543,84 +543,290 @@ class Horde_Config {
      */
     function _configSQL($ctx, $node = null, $switchname = 'driverconfig')
     {
-        $config =
-            array(
-                'desc' => 'Driver configuration',
-                'default' => $this->_default($ctx . '|' . (isset($node) ? $node->get_attribute('switchname') : $switchname), 'horde'),
-                'switch' => array(
-                    'horde' => array(
-                        'desc' => 'Horde defaults',
-                        'fields' => array()),
-                    'custom' => array(
-                        'desc' => 'Custom parameters',
-                        'fields' => array(
-                            md5(microtime()) => array(
-                                '_type' => 'description',
-                                'desc' => 'Any parameters that the storage driver needs.'),
-                            'phptype' => array(
-                                '_type' => 'enum',
-                                'required' => true,
-                                'quote' => true,
-                                'desc' => 'What database backend should we use?',
-                                'default' => $this->_default($ctx . '|phptype', 'mysql'),
-                                'values' => array(
-                                    'dbase' => 'dBase',
-                                    'ibase' => 'Firebird/InterBase',
-                                    'fbsql' => 'Frontbase',
-                                    'ifx' => 'Informix',
-                                    'msql' => 'mSQL',
-                                    'mssql' => 'MS SQL Server',
-                                    'mysql' => 'MySQL',
-                                    'oci8' => 'Oracle 8',
-                                    'odbc' => 'ODBC',
-                                    'pgsql' => 'PostgreSQL',
-                                    'sqlite' => 'SQLite',
-                                    'sybase' => 'Sybase')),
-                            'hostspec' => array(
-                                '_type' => 'text',
-                                'required' => true,
-                                'desc' => 'Database server/host/ODBC DSN',
-                                'default' => $this->_default($ctx . '|hostspec', 'localhost')),
-                            'username' => array(
-                                '_type' => 'text',
-                                'required' => true,
-                                'desc' => 'Username to connect to the database as',
-                                'default' => $this->_default($ctx . '|username', 'horde')),
-                            'password' => array(
-                                '_type' => 'text',
-                                'required' => false,
-                                'desc' => 'Password to connect with',
-                                'default' => $this->_default($ctx . '|password', '*****')),
-                            'database' => array(
-                                '_type' => 'text',
-                                'required' => true,
-                                'desc' => 'Database name to use',
-                                'default' => $this->_default($ctx . '|database', 'horde')),
-                            'protocol' => array(
-                                'desc' => 'How should we connect to the database?',
-                                'default' => $this->_default($ctx . '|protocol', 'unix'),
-                                'switch' => array(
-                                    'unix' => array(
-                                        'desc' => 'UNIX Sockets',
-                                        'fields' => array(
-                                            'socket' => array(
-                                                '_type' => 'text',
-                                                'required' => false,
-                                                'desc' => 'Location of UNIX socket, if using one',
-                                                'default' => $this->_default($ctx . '|socket', '')))),
-                                    'tcp' => array(
-                                        'desc' => 'TCP/IP',
-                                        'fields' => array(
-                                            'port' => array(
-                                                '_type' => 'int',
-                                                'required' => false,
-                                                'desc' => 'Port the DB is running on, if non-standard',
-                                                'default' => $this->_default($ctx . '|port', 3306)))))),
-                            'charset' => array(
-                                '_type' => 'text',
-                                'required' => true,
-                                'desc' => 'Internally used charset',
-                                'default' => $this->_default($ctx . '|charset', 'iso-8859-1'))))));
+        $persistent = array(
+            '_type' => 'boolean',
+            'required' => false,
+            'desc' => 'Request persistent connections?',
+            'default' => $this->_default($ctx . '|persistent', false));
+        $hostspec = array(
+            '_type' => 'text',
+            'required' => true,
+            'desc' => 'Database server/host',
+            'default' => $this->_default($ctx . '|hostspec', 'localhost'));
+        $username = array(
+            '_type' => 'text',
+            'required' => true,
+            'desc' => 'Username to connect to the database as',
+            'default' => $this->_default($ctx . '|username', 'horde'));
+        $password = array(
+            '_type' => 'text',
+            'required' => false,
+            'desc' => 'Password to connect with',
+            'default' => $this->_default($ctx . '|password', '*****'));
+        $database = array(
+            '_type' => 'text',
+            'required' => true,
+            'desc' => 'Database name to use',
+            'default' => $this->_default($ctx . '|database', 'horde'));
+        $socket = array(
+            '_type' => 'text',
+            'required' => false,
+            'desc' => 'Location of UNIX socket',
+            'default' => $this->_default($ctx . '|socket', ''));
+        $port = array(
+            '_type' => 'int',
+            'required' => false,
+            'desc' => 'Port the DB is running on, if non-standard',
+            'default' => $this->_default($ctx . '|port', null));
+        $protocol = array(
+            'desc' => 'How should we connect to the database?',
+            'default' => $this->_default($ctx . '|protocol', 'unix'),
+            'switch' => array(
+                'unix' => array(
+                    'desc' => 'UNIX Sockets',
+                    'fields' => array(
+                        'socket' => $socket)),
+                'tcp' => array(
+                    'desc' => 'TCP/IP',
+                    'fields' => array(
+                        'port' => $port))));
+        $mysql_protocol = $protocol;
+        $mysql_protocol['switch']['tcp']['fields']['port']['default'] = $this->_default($ctx . '|port', 3306);
+        $charset = array(
+            '_type' => 'text',
+            'required' => true,
+            'desc' => 'Internally used charset',
+            'default' => $this->_default($ctx . '|charset', 'iso-8859-1'));
+
+        $oci8_fields = array(
+            'persistent' => $persistent,
+            'username' => $username,
+            'password' => $password);
+        if (function_exists('oci_connect')) {
+            $oci8_fields['database'] = array(
+                '_type' => 'text',
+                'required' => true,
+                'desc' => 'Database name or Easy Connect parameter',
+                'default' => $this->_default($ctx . '|database', 'horde'));
+        } else {
+            $oci8_fields['hostspec'] = array(
+                '_type' => 'text',
+                'required' => true,
+                'desc' => 'Database name or Easy Connect parameter',
+                'default' => $this->_default($ctx . '|hostspec', 'horde'));
+        }
+        $oci8_fields['charset'] = $charset;
+
+        $custom_fields = array(
+            'required' => true,
+            'desc' => 'What database backend should we use?',
+            'default' => $this->_default($ctx . '|phptype', 'mysql'),
+            'switch' => array(
+                'dbase' => array(
+                    'desc' => 'dBase',
+                    'fields' => array(
+                        'database' => array(
+                            '_type' => 'text',
+                            'required' => true,
+                            'desc' => 'Absolute path to the database file',
+                            'default' => $this->_default($ctx . '|database', '')),
+                        'mode' => array(
+                            '_type' => 'enum',
+                            'desc' => 'The mode to open the file with',
+                            'values' => array(
+                                0 => 'Read Only',
+                                2 => 'Read Write'),
+                            'default' => $this->_default($ctx . '|mode', 2)),
+                        
+                        'charset' => $charset)),
+                'ibase' => array(
+                    'desc' => 'Firebird/InterBase',
+                    'fields' => array(
+                        'dbsyntax' => array(
+                            '_type' => 'enum',
+                            'desc' => 'The database syntax variant to use',
+                            'required' => false,
+                            'values' => array(
+                                'ibase' => 'InterBase',
+                                'firebird' => 'Firebird'),
+                            'default' => $this->_default($ctx . '|dbsyntax', 'firebird')),
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'database' => $database,
+                        'buffers' => array(
+                            '_type' => 'int',
+                            'desc' => 'The number of database buffers to allocate',
+                            'required' => false,
+                            'default' => $this->_default($ctx . '|buffers', null)),
+                        'dialect' => array(
+                            '_type' => 'int',
+                            'desc' => 'The default SQL dialect for any statement executed within a connection.',
+                            'required' => false,
+                            'default' => $this->_default($ctx . '|dialect', null)),
+                        'role' => array(
+                            '_type' => 'text',
+                            'desc' => 'Role',
+                            'required' => false,
+                            'default' => $this->_default($ctx . '|role', null)),
+                        'charset' => $charset)),
+                'fbsql' => array(
+                    'desc' => 'Frontbase',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'ifx' => array(
+                    'desc' => 'Informix',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'username' => $username,
+                        'password' => $password,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'msql' => array(
+                    'desc' => 'mSQL',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'port' => $port,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'mssql' => array(
+                    'desc' => 'MS SQL Server',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'port' => $port,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'mysql' => array(
+                    'desc' => 'MySQL',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'protocol' => $mysql_protocol,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'mysqli' => array(
+                    'desc' => 'MySQL improved (4.1+)',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'port' => $port,
+                        'socket' => $socket,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'oci8' => array(
+                    'desc' => 'Oracle',
+                    'fields' => $oci8_fields),
+                'odbc' => array(
+                    'desc' => 'ODBC',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'username' => $username,
+                        'password' => $password,
+                        'hostspec' => array(
+                            '_type' => 'text',
+                            'desc' => 'DSN',
+                            'default' => $this->_default($ctx . '|hostspec', '')),
+                        'dbsyntax' => array(
+                            '_type' => 'enum',
+                            'desc' => 'The database syntax variant to use',
+                            'required' => false,
+                            'values' => array(
+                                'sql92' => 'SQL92',
+                                'access' => 'Access',
+                                'db2' => 'DB2',
+                                'solid' => 'Solid',
+                                'navision' => 'Navision',
+                                'mssql' => 'MS SQL Server',
+                                'sybase' => 'Sybase',
+                                'mysql' => 'MySQL',
+                                'mysqli' => 'MySQL improved',
+                                ),
+                            'default' => $this->_default($ctx . '|dbsyntax', 'sql92')),
+                        'cursor' => array(
+                            '_type' => 'enum',
+                            'desc' => 'Cursor type',
+                            'quote' => false,
+                            'required' => false,
+                            'values' => array(
+                                'null' => 'None',
+                                'SQL_CUR_DEFAULT' => 'Default',
+                                'SQL_CUR_USE_DRIVER' => 'Use Driver',
+                                'SQL_CUR_USE_ODBC' => 'Use ODBC',
+                                'SQL_CUR_USE_IF_NEEDED' => 'Use If Needed'),
+                            'default' => $this->_default($ctx . '|cursor', null)),
+                        'charset' => $charset)),
+                'pgsql' => array(
+                    'desc' => 'PostgreSQL',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'protocol' => $protocol,
+                        'database' => $database,
+                        'charset' => $charset)),
+                'sqlite' => array(
+                    'desc' => 'SQLite',
+                    'fields' => array(
+                        'database' => array(
+                            '_type' => 'text',
+                            'required' => true,
+                            'desc' => 'Absolute path to the database file',
+                            'default' => $this->_default($ctx . '|database', '')),
+                        'mode' => array(
+                            '_type' => 'octal',
+                            'desc' => 'The mode to open the file with',
+                            'default' => $this->_default($ctx . '|mode', '0644')),
+                        'charset' => $charset)),
+                'sybase' => array(
+                    'desc' => 'Sybase',
+                    'fields' => array(
+                        'persistent' => $persistent,
+                        'hostspec' => $hostspec,
+                        'username' => $username,
+                        'password' => $password,
+                        'database' => $database,
+                        'appname' => array(
+                            '_type' => 'text',
+                            'desc' => 'Application Name',
+                            'required' => false,
+                            'default' => $this->_default($ctx . '|appname', '')),
+                        'charset' => $charset))));
+
+        if (isset($node) && $node->get_attribute('baseconfig') == 'true') {
+            return $custom_fields;
+        }
+
+        $config = array(
+            'desc' => 'Driver configuration',
+            'default' => $this->_default($ctx . '|' . (isset($node) ? $node->get_attribute('switchname') : $switchname), 'horde'),
+            'switch' => array(
+                'horde' => array(
+                    'desc' => 'Horde defaults',
+                    'fields' => array()),
+                'custom' => array(
+                    'desc' => 'Custom parameters',
+                    'fields' => array(
+                        md5(microtime()) => array(
+                            '_type' => 'description',
+                            'desc' => 'Any parameters that the storage driver needs.'),
+                        'phptype' => $custom_fields))));
 
         if (isset($node) && $node->has_child_nodes()) {
             $cur = array();
