@@ -3,7 +3,7 @@
  * The MIME_Viewer_html class renders out HTML text with an effort to
  * remove potentially malicious code.
  *
- * $Horde: framework/MIME/MIME/Viewer/html.php,v 1.14.4.3 2005/02/05 17:49:01 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer/html.php,v 1.14.4.6 2005/03/20 18:39:44 jan Exp $
  *
  * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
@@ -15,7 +15,7 @@
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @author  Jon Parise <jon@horde.org>
  * @author  Michael Slusarz <slusarz@horde.org>
- * @version $Revision: 1.14.4.3 $
+ * @version $Revision: 1.14.4.6 $
  * @since   Horde 3.0
  * @package Horde_MIME_Viewer
  */
@@ -145,7 +145,7 @@ class MIME_Viewer_html extends MIME_Viewer {
         /* Get all tags that might cause trouble - <object>, <embed>,
          * <base>, etc. Meta refreshes and iframes, too. */
         $malicious = array(
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(s|&#0*83;?|&#x0*53;?|&#0*115;?|&#x0*73;?)\s*' .
             '(c|&#0*67;?|&#x0*43;?|&#0*99;?|&#x0*63;?)\s*' .
             '(r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
@@ -153,33 +153,33 @@ class MIME_Viewer_html extends MIME_Viewer {
             '(p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
             '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*/i',
 
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
             '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
             '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
             '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
             '(d|&#0*68;?|&#0*44;?|&#0*100;?|&#0*64;?)\s*/i',
 
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
             '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
             '(s|&#0*83;?|&#x0*53;?|&#0*115;?|&#x0*73;?)\s*' .
             '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
             '[^line]/i',
 
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
             '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
             '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*' .
             '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*/i',
 
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(j|&#0*74;?|&#0*4a;?|&#0*106;?|&#0*6a;?)\s*' .
             '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
             '(v|&#0*86;?|&#0*56;?|&#0*118;?|&#0*76;?)\s*' .
             '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*/i',
 
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(o|&#0*79;?|&#0*4f;?|&#0*111;?|&#0*6f;?)\s*' .
             '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
             '(j|&#0*74;?|&#0*4a;?|&#0*106;?|&#0*6a;?)\s*' .
@@ -187,7 +187,7 @@ class MIME_Viewer_html extends MIME_Viewer {
             '(c|&#0*67;?|&#x0*43;?|&#0*99;?|&#x0*63;?)\s*' .
             '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*/i',
 
-            '/<([^>]*)' .
+            '/<([^>a-z]*)' .
             '(i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)\s*' .
             '(f|&#0*70;?|&#0*46;?|&#0*102;?|&#0*66;?)\s*' .
             '(r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
@@ -239,7 +239,7 @@ class MIME_Viewer_html extends MIME_Viewer {
 
         /* Try to derefer all external references. */
         $data = preg_replace_callback('/href\s*=\s*(["\'])?((?(1)[^\1]*?|[^\s]+))(?(1)\1|)/i',
-                                      create_function('$m', 'return \'href="\' . Horde::externalUrl($m[2]) . \'"\';'),
+                                      create_function('$m', 'return \'href="\' . (strlen($m[2]) && $m[2]{0} == \'#\' ? $m[2] : Horde::externalUrl($m[2])) . \'"\';'),
                                       $data);
 
         return $data;
