@@ -39,7 +39,7 @@
  *   $conf['prefs']['params']['password'] = 'password';
  *
  *
- * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.3 2005/02/09 21:39:51 chuck Exp $
+ * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.4 2005/03/03 23:04:35 chuck Exp $
  *
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
  *
@@ -136,7 +136,7 @@ class Prefs_ldap extends Prefs {
         }
 
         Horde::assertDriverConfig($this->_params, 'prefs',
-            array('hostspec', 'basedn', 'uid', 'rootdn', 'password'),
+            array('hostspec', 'basedn', 'uid', 'password'),
             'preferences LDAP');
 
         /* Allow for multiple uid search fields */
