@@ -2,7 +2,7 @@
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.4 2005/02/06 19:25:26 chuck Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.7 2005/03/09 11:26:25 jan Exp $
  *
  * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
  *
@@ -95,7 +95,7 @@ class Horde_iCalendar {
         $keys = array_keys($this->_attributes);
         foreach ($keys as $key) {
             if ($found) break;
-            if ($this->_attributes[$key]['name'] == $name) {
+            if ($this->_attributes[$key]['name'] == String::upper($name)) {
                 $this->_attributes[$key]['params'] = $params;
                 $this->_attributes[$key]['value'] = $value;
                 $this->_attributes[$key]['values'] = $values;
@@ -105,7 +105,7 @@ class Horde_iCalendar {
 
         if ($append || !$found) {
             $this->_attributes[] = array(
-                'name'      => $name,
+                'name'      => String::upper($name),
                 'params'    => $params,
                 'value'     => $value,
                 'values'    => $values
@@ -160,7 +160,7 @@ class Horde_iCalendar {
                 }
             }
         }
-        if (count($result) == 0) {
+        if (!count($result)) {
             require_once 'PEAR.php';
             return PEAR::raiseError('Attribute "' . $name . '" Not Found');
         } if (count($result) == 1 && !$params) {
@@ -363,7 +363,7 @@ class Horde_iCalendar {
     {
         // Default values.
         $requiredAttributes['VERSION'] = '2.0';
-        $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library, Horde 3.0-cvs //EN';
+        $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library' . (defined('HORDE_VERSION') ? ', Horde ' . constant('HORDE_VERSION') : '') . ' //EN';
         $requiredAttributes['METHOD'] = 'PUBLISH';
 
         foreach ($requiredAttributes as $name => $default_value) {
@@ -429,7 +429,7 @@ class Horde_iCalendar {
 
         // All subcomponents.
         $matches = null;
-        if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/U', $vCal, $matches)) {
+        if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/Ui', $vCal, $matches)) {
             foreach ($matches[0] as $key => $data) {
                 $type = $matches[1][$key];
                 $component = &Horde_iCalendar::newComponent(trim($type), $this);
@@ -538,17 +538,9 @@ class Horde_iCalendar {
                     preg_match_all('/,([^,]*)/', ',' . $value, $values);
 
                     foreach ($values[1] as $value) {
-                        if (isset($params['VALUE'])) {
-                            if ($params['VALUE'] == 'DATE-TIME') {
-                                $dates[] = $this->_parseDateTime($value);
-                            } elseif ($params['VALUE'] == 'DATE') {
-                                $dates[] = $this->_parseDate($value);
-                            }
-                        } else {
-                            $dates[] = $this->_parseDateTime($value);
-                        }
+                        $dates[] = $this->_parseDate($value);
                     }
-                    $this->setAttribute($tag, $dates, $params);
+                    $this->setAttribute($tag, isset($dates[0]) ? $dates[0] : null, $params, true, $dates);
                     break;
 
                 // Duration fields.
@@ -564,7 +556,7 @@ class Horde_iCalendar {
                         $periods[] = $this->_parsePeriod($value);
                     }
 
-                    $this->setAttribute($tag, $periods, $params);
+                    $this->setAttribute($tag, isset($periods[0]) ? $periods[0] : null, $params, true, $periods);
                     break;
 
                 // UTC offset fields.
@@ -781,7 +773,7 @@ class Horde_iCalendar {
         }
 
         foreach ($this->_components as $component) {
-            $result .= $component->exportvCalendar() . $this->_newline;
+            $result .= $component->exportvCalendar();
         }
 
         return $result . 'END:' . $base . $this->_newline;
@@ -975,15 +967,18 @@ class Horde_iCalendar {
      */
     function _parseDate($text)
     {
+        $parts = explode('T', $text);
+        if (count($parts) == 2) {
+            $text = $parts[0];
+        }
+
         if (strlen($text) != 8) {
             return false;
         }
 
-        $date['year']  = intval(substr($text, 0, 4));
-        $date['month'] = intval(substr($text, 4, 2));
-        $date['mday']  = intval(substr($text, 6, 2));
-
-        return $date;
+        return array('year' => intval(substr($text, 0, 4)),
+                     'month' => intval(substr($text, 4, 2)),
+                     'mday' => intval(substr($text, 6, 2)));
     }
 
     /**
