@@ -16,7 +16,7 @@
  *                       negotiated with the server.
  * </pre>
  *
- * $Horde: framework/VFS/VFS/ftp.php,v 1.75.4.3 2005/01/31 09:25:27 jan Exp $
+ * $Horde: framework/VFS/VFS/ftp.php,v 1.75.4.4 2005/02/17 22:09:10 jan Exp $
  *
  * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2002-2005 Michael Varghese <mike.varghese@ascellatech.com>
@@ -79,15 +79,21 @@ class VFS_ftp extends VFS {
             return PEAR::raiseError(sprintf(_("Unable to open VFS file \"%s\"."), $this->_getPath($path, $name)));
         }
 
+        $size = filesize($tmpFile);
+        if ($size === 0) {
+            return '';
+        }
+
         if (OS_WINDOWS) {
             $mode = 'rb';
         } else {
             $mode = 'r';
         }
         $fp = fopen($tmpFile, $mode);
-        $data = fread($fp, filesize($tmpFile));
+        $data = fread($fp, $size);
         fclose($fp);
         unlink($tmpFile);
+
         return $data;
     }
 
