@@ -2,10 +2,17 @@
 
 require_once dirname(__FILE__) . '/sql.php';
 
+/** @constant integer VFS_FLAG_READ  Permission for read access. */
+define('VFS_FLAG_READ', 1);
+
+/** @constant integer VFS_FLAG_WRITE  Permission for read access. */
+define('VFS_FLAG_WRITE', 2);
+
 /**
- * Multi User Horde_VFS implementation for PHP's PEAR database
+ * Multi User VFS implementation for PHP's PEAR database
  * abstraction layer.
  *
+ * <pre>
  * Required values for $params:
  *      'phptype'       The database type (ie. 'pgsql', 'mysql, etc.).
  *      'hostspec'      The hostname of the database server.
@@ -15,16 +22,19 @@ require_once dirname(__FILE__) . '/sql.php';
  *      'database'      The name of the database.
  *
  * Optional values:
- *      'table'         The name of the vfs table in 'database'. Defaults to 'horde_muvfs'.
+ *      'table'         The name of the vfs table in 'database'. Defaults to
+ *                      'horde_muvfs'.
  *
  * Required by some database implementations:
  *      'options'       Additional options to pass to the database.
  *      'tty'           The TTY on which to connect to the database.
  *      'port'          The port on which to connect to the database.
+ * </pre>
  *
  * Known Issues:
- * Delete is not recusive, so file and folders that used to be in a folder that gets
- * deleted life forever in the database, or re-appear when the folder is recreated.
+ * Delete is not recusive, so file and folders that used to be in a folder that
+ * gets deleted life forever in the database, or re-appear when the folder is
+ * recreated.
  * Rename has the same issue, if files are lost if a folder is renamed.
  *
  * The table structure for the VFS can be found in
@@ -35,49 +45,49 @@ require_once dirname(__FILE__) . '/sql.php';
  * MSSQL:
  * - The vfs_data field must be of type IMAGE.
  * - You need the following php.ini settings:
+ * <code>
  *    ; Valid range 0 - 2147483647. Default = 4096.
  *    mssql.textlimit = 0 ; zero to pass through
  *
  *    ; Valid range 0 - 2147483647. Default = 4096.
  *    mssql.textsize = 0 ; zero to pass through
+ * </code>
  *
- * $Horde: horde/lib/VFS/musql.php,v 1.4.2.4 2003/01/03 13:23:18 jan Exp $
+ * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.1 2005/01/03 12:19:20 jan Exp $
  *
- * Copyright 2002-2003 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.4.2.4 $
+ * @version $Revision: 1.30.4.1 $
  * @since   Horde 2.2
- * @package horde.vfs
+ * @package VFS
  */
-
-/** @const HORDE_VFS_FLAG_READ Permission for read access. */
-define('HORDE_VFS_FLAG_READ', 1);
-
-/** @const HORDE_VFS_FLAG_WRITE Permission for read access. */
-define('HORDE_VFS_FLAG_WRITE', 2);
-
-class Horde_VFS_musql extends Horde_VFS_sql {
+class VFS_musql extends VFS_sql {
 
     /**
      * List of permissions and if they can be changed in this VFS
+     *
+     * @var array $_permissions
      */
     var $_permissions = array(
         'owner' => array('read' => false, 'write' => false, 'execute' => false),
         'group' => array('read' => false, 'write' => false, 'execute' => false),
-        'all'   => array('read' => true,  'write' => true,  'execute' => false));
+        'all'   => array('read' => true,  'write' => true,  'execute' => false)
+    );
 
     /**
      * Store a file in the VFS from raw data.
      *
-     * @param string  $path        The path to store the file in.
-     * @param string  $name        The filename to use.
-     * @param string  $data        The file data.
-     * @param boolean $autocreate  (optional) Automatically create directories?
+     * @access public
+     *
+     * @param string $path                  The path to store the file in.
+     * @param string $name                  The filename to use.
+     * @param string $data                  The file data.
+     * @param optional boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -92,6 +102,18 @@ class Horde_VFS_musql extends Horde_VFS_sql {
          * paths. */
         if ($path != '') {
             $paths = explode('/', $path);
+            $path_name = array_pop($paths);
+            if (!$this->isFolder(implode('/', $paths), $path_name)) {
+                if (!$autocreate) {
+                    return PEAR::raiseError(sprintf(_("Folder %s does not exist"), $path), 'horde.error');
+                } else {
+                    $result = $this->autocreatePath($path);
+                    if (is_a($result, 'PEAR_Error')) {
+                        return $result;
+                    }
+                }
+            }
+            $paths[] = $path_name;
             $previous = '';
 
             foreach ($paths as $thispath) {
@@ -101,19 +123,19 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                                       $this->_db->quote($previous),
                                                       $this->_db->quote($thispath)));
                 if (!is_array($results) || count($results) < 1) {
-                    return PEAR::raiseError('Unable to create VFS file.');
+                    return PEAR::raiseError(_("Unable to create VFS file."));
                 }
 
                 $allowed = false;
                 foreach ($results as $result) {
-                    if ($result[0] == Auth::getAuth() || $result[1] & HORDE_VFS_FLAG_WRITE) {
+                    if ($result[0] == $this->_params['user'] || $result[1] & VFS_FLAG_WRITE) {
                         $allowed = true;
                         break;
                     }
                 }
 
                 if (!$allowed) {
-                    return PEAR::raiseError('Access denied creating VFS file.');
+                    return PEAR::raiseError(_("Access denied creating VFS file."));
                 }
 
                 $previous = $thispath;
@@ -126,6 +148,8 @@ class Horde_VFS_musql extends Horde_VFS_sql {
     /**
      * Delete a file from the VFS.
      *
+     * @access public
+     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      *
@@ -143,35 +167,37 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                                $this->_params['table'],
                                                $this->_db->quote($path),
                                                $this->_db->quote($name),
-                                               $this->_db->quote(HORDE_VFS_FILE)));
+                                               $this->_db->quote(VFS_FILE)));
 
         if (!is_array($fileList) || count($fileList) < 1) {
-            return PEAR::raiseError('Unable to delete VFS file.');
+            return PEAR::raiseError(_("Unable to delete VFS file."));
         }
 
         /* There may be one or more files with the same name but the
            user may not have read access to them, so doesn't see
            them. So we have to delete the one they have access to. */
         foreach ($fileList as $file) {
-            if ($file[1] == Auth::getAuth() || $file[2] & HORDE_VFS_FLAG_WRITE) {
+            if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
                 $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = %s',
                                                     $this->_params['table'],
                                                     $this->_db->quote($file[0])));
 
                 if ($this->_db->affectedRows() == 0) {
-                    return PEAR::raiseError('Unable to delete VFS file.');
+                    return PEAR::raiseError(_("Unable to delete VFS file."));
                 }
                 return $result;
             }
         }
 
         // FIXME: 'Access Denied deleting file %s/%s'
-        return PEAR::raiseError('Unable to delete VFS file.');
+        return PEAR::raiseError(_("Unable to delete VFS file."));
     }
 
     /**
      * Rename a file or folder in the VFS.
      *
+     * @access public
+     *
      * @param string $oldpath  The old path to the file.
      * @param string $oldname  The old filename.
      * @param string $newpath  The new path of the file.
@@ -193,14 +219,14 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                                        $this->_db->quote($oldname)));
 
         if (!is_array($fileList) || count($fileList) < 1) {
-            return PEAR::raiseError('Unable to rename VFS file.');
+            return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
-        /* There may be one or more files with the same name but the user may not have
-           read access to them, so doesn't see them. So we have to rename the one they have
-           access to. */
+        /* There may be one or more files with the same name but the user may
+	 * not have read access to them, so doesn't see them. So we have to
+	 * rename the one they have access to. */
         foreach ($fileList as $file) {
-            if ($file[1] == Auth::getAuth() || $file[2] & HORDE_VFS_FLAG_WRITE) {
+            if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
                 $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = %s, vfs_name = %s, vfs_modified = %s
                                                     WHERE vfs_id = %s',
                                                     $this->_params['table'],
@@ -212,14 +238,16 @@ class Horde_VFS_musql extends Horde_VFS_sql {
             }
         }
 
-        return PEAR::raiseError(sprintf(_('Unable to rename VFS file %s/%s.'), $oldpath, $oldname));
+        return PEAR::raiseError(sprintf(_("Unable to rename VFS file %s/%s."), $oldpath, $oldname));
     }
 
     /**
      * Creates a folder on the VFS.
      *
-     * @param string $path Holds the path of directory to create folder.
-     * @param string $name Holds the name of the new folder.
+     * @access public
+     *
+     * @param string $path  Holds the path of directory to create folder.
+     * @param string $name  Holds the name of the new folder.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -242,19 +270,19 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                                                $this->_db->quote($previous),
                                                                $this->_db->quote($thispath)));
                 if (!is_array($results) || count($results) < 1) {
-                    return PEAR::raiseError('Unable to create VFS folder.');
+                    return PEAR::raiseError(_("Unable to create VFS directory."));
                 }
 
                 $allowed = false;
                 foreach ($results as $result) {
-                    if ($result[0] == Auth::getAuth() || $result[1] & HORDE_VFS_FLAG_WRITE) {
+                    if ($result[0] == $this->_params['user'] || $result[1] & VFS_FLAG_WRITE) {
                         $allowed = true;
                         break;
                     }
                 }
 
                 if (!$allowed) {
-                    return PEAR::raiseError('Access denied creating VFS folder.');
+                    return PEAR::raiseError(_("Access denied creating VFS directory."));
                 }
 
                 $previous = $thispath;
@@ -266,11 +294,11 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                          VALUES (%s, %s, %s, %s, %s, %s, %s)',
                                          $this->_params['table'],
                                          $this->_db->quote($id),
-                                         $this->_db->quote(HORDE_VFS_FOLDER),
+                                         $this->_db->quote(VFS_FOLDER),
                                          $this->_db->quote($path),
                                          $this->_db->quote($name),
                                          $this->_db->quote(time()),
-                                         $this->_db->quote(Auth::getAuth()),
+                                         $this->_db->quote($this->_params['user']),
                                          $this->_db->quote(0)
                                          ));
     }
@@ -278,40 +306,59 @@ class Horde_VFS_musql extends Horde_VFS_sql {
     /**
      * Delete a file from the VFS.
      *
-     * @param string $path  The path to store the file in.
-     * @param string $name  The filename to use.
+     * @access public
+     *
+     * @param string $path                 The path to delete the folder from.
+     * @param string $name                 The foldername to use.
+     * @param optional boolean $recursive  Force a recursive delete?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function deleteFolder($path, $name)
+    function deleteFolder($path, $name, $recursive = false)
     {
         $conn = $this->_connect();
         if (PEAR::isError($conn)) {
             return $conn;
         }
 
+        if ($recursive) {
+            $result = $this->emptyFolder($path . '/' . $name);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        } else {
+            $list = $this->listFolder($path . '/' . $name);
+            if (is_a($list, 'PEAR_Error')) {
+                return $list;
+            }
+            if (count($list)) {
+                return PEAR::raiseError(sprintf(_("Unable to delete %s, the directory is not empty"),
+                                                $path . '/' . $name));
+            }
+        }
+
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_id, vfs_owner, vfs_perms FROM %s
                                                        WHERE vfs_path = %s AND vfs_name= %s AND vfs_type = %s',
                                                        $this->_params['table'],
                                                        $this->_db->quote($path),
                                                        $this->_db->quote($name),
-                                                       $this->_db->quote(HORDE_VFS_FOLDER)));
+                                                       $this->_db->quote(VFS_FOLDER)));
 
         if (!is_array($fileList) || count($fileList) < 1) {
-            return PEAR::raiseError('Unable to delete VFS directory.');
+            return PEAR::raiseError(_("Unable to delete VFS directory."));
         }
 
-        /* There may be one or more folders with the same name but as the user may not have
-           read access to them, they don't see them. So we have to delete the one they have
-           access to */
+        /* There may be one or more folders with the same name but as the user
+	 * may not have read access to them, they don't see them. So we have to
+	 * delete the one they have access to */
         foreach ($fileList as $file) {
-            if ($file[1] == Auth::getAuth() || $file[2] & HORDE_VFS_FLAG_WRITE) {
+            if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
                 $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = %s',
                                                     $this->_params['table'],
                                                     $this->_db->quote($file[0])));
 
                 if ($this->_db->affectedRows() == 0) {
-                    return PEAR::raiseError('Unable to delete VFS directory.');
+                    return PEAR::raiseError(_("Unable to delete VFS directory."));
                 }
 
                 return $result;
@@ -319,20 +366,25 @@ class Horde_VFS_musql extends Horde_VFS_sql {
         }
 
         // FIXME: 'Access Denied deleting folder %s/%s'
-        return PEAR::raiseError('Unable to delete VFS directory.');
+        return PEAR::raiseError(_("Unable to delete VFS directory."));
     }
 
     /**
      * Return a list of the contents of a folder.
      *
-     * @param string $path Holds the path of the directory.
+     * @param string $path                The path of the directory.
+     * @param optional mixed $filter      String/hash to filter file/dirname
+     *                                    on.
+     * @param optional boolean $dotfiles  Show dotfiles?
+     * @param optional boolean $dironly   Show only directories?
      *
      * @return mixed  File list on success or false on failure.
      */
-    function listFolder($path)
+    function _listFolder($path, $filter = null, $dotfiles = true,
+                        $dironly = false)
     {
         $conn = $this->_connect();
-        if (PEAR::isError($conn)) {
+        if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
@@ -343,41 +395,59 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                                WHERE vfs_path = %s AND (vfs_owner = %s or vfs_perms && %s)',
                                                $this->_params['table'],
                                                $this->_db->quote($path),
-                                               $this->_db->quote(Auth::getAuth()),
-                                               $this->_db->quote(HORDE_VFS_FLAG_READ)
+                                               $this->_db->quote($this->_params['user']),
+                                               $this->_db->quote(VFS_FLAG_READ)
                                                ));
+        if (is_a($fileList, 'PEAR_Error')) {
+            return $fileList;
+        }
+
         foreach ($fileList as $line) {
+            // Filter out dotfiles if they aren't wanted.
+            if (!$dotfiles && substr($line[0], 0, 1) == '.') {
+                continue;
+            }
+
             $file['name'] = stripslashes($line[0]);
 
-            if ($line[1] == HORDE_VFS_FILE) {
+            if ($line[1] == VFS_FILE) {
                 $name = explode('.', $line[0]);
 
                 if (count($name) == 1) {
                     $file['type'] = '**none';
                 } else {
-                    $file['type'] = strtolower($name[count($name) - 1]);
+                    $file['type'] = VFS::strtolower($name[count($name) - 1]);
                 }
 
-                $file['size'] = strlen($line[5]);
-            } else if ($line[1] == HORDE_VFS_FOLDER) {
+                $file['size'] = VFS::strlen($line[5]);
+            } elseif ($line[1] == VFS_FOLDER) {
                 $file['type'] = '**dir';
                 $file['size'] = -1;
             }
 
             $file['date'] = $line[2];
-
             $file['owner'] = $line[3];
 
             $line[4] = intval($line[4]);
-
-            $file['perms']  = ($line[1] == HORDE_VFS_FOLDER) ? 'd' : '-';
+            $file['perms']  = ($line[1] == VFS_FOLDER) ? 'd' : '-';
             $file['perms'] .= 'rw-';
-            $file['perms'] .= ($line[4] & HORDE_VFS_FLAG_READ) ? 'r' : '-';
-            $file['perms'] .= ($line[4] & HORDE_VFS_FLAG_WRITE) ? 'w' : '-';
+            $file['perms'] .= ($line[4] & VFS_FLAG_READ) ? 'r' : '-';
+            $file['perms'] .= ($line[4] & VFS_FLAG_WRITE) ? 'w' : '-';
             $file['perms'] .= '-';
             $file['group'] = '-';
 
+            // filtering
+            if ($this->_filterMatch($filter, $file['name'])) {
+                unset($file);
+                continue;
+            }
+            if ($dironly && $file['type'] !== '**dir') {
+                unset($file);
+                continue;
+            }
+
             $files[$file['name']] = $file;
+            unset($file);
         }
 
         return $files;
@@ -386,8 +456,10 @@ class Horde_VFS_musql extends Horde_VFS_sql {
     /**
      * Changes permissions for an Item on the VFS.
      *
-     * @param string $path Holds the path of directory of the Item.
-     * @param string $name Holds the name of the Item.
+     * @access public
+     *
+     * @param string $path  Holds the path of directory of the Item.
+     * @param string $name  Holds the name of the Item.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -400,8 +472,8 @@ class Horde_VFS_musql extends Horde_VFS_sql {
 
         $val = intval(substr($permission, -1));
         $perm = 0;
-        $perm |= ($val & 4) ? HORDE_VFS_FLAG_READ : 0;
-        $perm |= ($val & 2) ? HORDE_VFS_FLAG_WRITE : 0;
+        $perm |= ($val & 4) ? VFS_FLAG_READ : 0;
+        $perm |= ($val & 2) ? VFS_FLAG_WRITE : 0;
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_id, vfs_owner, vfs_perms FROM %s
                                                        WHERE vfs_path = %s AND vfs_name= %s',
@@ -410,14 +482,14 @@ class Horde_VFS_musql extends Horde_VFS_sql {
                                                        $this->_db->quote($name)));
 
         if (!is_array($fileList) || count($fileList) < 1) {
-            return PEAR::raiseError('Unable to rename VFS file.');
+            return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
-        /* There may be one or more files with the same name but the user may not have
-           read access to them, so doesn't see them. So we have to chmod the one they have
-           access to. */
+        /* There may be one or more files with the same name but the user may
+	 * not have read access to them, so doesn't see them. So we have to
+	 * chmod the one they have access to. */
         foreach ($fileList as $file) {
-            if ($file[1] == Auth::getAuth() || $file[2] & HORDE_VFS_FLAG_WRITE) {
+            if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
                 $result = $this->_db->query(sprintf('UPDATE %s SET vfs_perms = %s
                                                     WHERE vfs_id = %s',
                                                     $this->_params['table'],
@@ -427,7 +499,7 @@ class Horde_VFS_musql extends Horde_VFS_sql {
             }
         }
 
-        return PEAR::raiseError(sprintf(_('Unable to change permission for VFS file %s/%s.'), $path, $name));
+        return PEAR::raiseError(sprintf(_("Unable to change permission for VFS file %s/%s."), $path, $name));
     }
 
 }
